c basissp.f, (Aug 12 1995)
c This file contains function FNULL to compute all the null space 
c basis functions and function RK to compute reproducing kernel,
c for pseudo thin plate splines on sphere.
c ----------------------------------------------------------------------
c   null space basis functions 
      DOUBLE PRECISION FUNCTION FNULL (M,X1,X2)
      INTEGER M
      DOUBLE PRECISION X1,X2
      IF(M .EQ. 1) THEN
         FNULL = 1.D0
      ELSE
         WRITE(*,*) "Error: null space basis function inconsistency"
      ENDIF
      RETURN
      END

c   reproducing kernel 
C   reproducing kernel from wahba (1981) SIAM J.SCI.STAT. COMPUT.
C   (3.3) and (3.4) with m = 2, for splines on the sphere.
C   - 90 < X < 90, -180 < Y < 180.
      DOUBLE PRECISION FUNCTION  RK(X1,Y1,X2,Y2)
      DOUBLE PRECISION X1,X2,Y1,Y2, Z, PI
      DATA PI /3.141593D0/
      RK = Z(X1,Y1,X2,Y2)
c (3.4) with m = 2
      IF (RK .GT. 0.999999D0) THEN
         RK = .5D0
      ELSE
         RK = LOG(1.D0 + SQRT(2.D0/(1.D0-RK)))
     &        *(12.D0*(.5D0-RK/2.D0)**2-2.D0+2.D0*RK)
     &        -12.D0*SQRT(.5D0-RK/2.D0)**3
     &        +3.D0-3.D0*RK + 1.D0
         RK = RK/2.D0
      ENDIF
c (3.3)
      RK = RK/2.D0 - 1.D0/6.D0
      RK = RK/PI/2.D0
      RETURN
      END

c A function to calculate the cosine of the angle between two points on
c the sphere
C  -90. < LAT < 90.,    -180. < LONG < 180.
      DOUBLE PRECISION FUNCTION Z(LAT1,LONG1,LAT2,LONG2)
      DOUBLE PRECISION LAT1,LONG1,LAT2,LONG2, PI, THETA
      PARAMETER (PI = 3.141593D0)
      THETA = (LONG1-LONG2)/180.D0*PI
      Z = COS(LAT1/180.D0*PI)*COS(LAT2/180.D0*PI)*COS(THETA) +
     &    SIN(LAT1/180.D0*PI)*SIN(LAT2/180.D0*PI)
      RETURN
      END

